# Mime Normalizer

Library implements normalisation of HTML and plain parts of a MIME message.

Normalisation strategies documentation:
https://code.vereign.com/light/documentation/-/blob/master/Validation.md#normalization-strategies

## Prerequisites

- [Node.JS](https://nodejs.org/en/) ^10.12.0 || >=12.0.0+
- [yarn](https://yarnpkg.com/getting-started/install) package manager

## Initialize project

`yarn install`

## Testing

### Running

- `yarn test` - runs tests once
- `yarn test:watch` - runs tests in watch mode
  - use `p` key to run cases by regular expression, e.g.
    `html-`
    `.*-outlook-outlook`
    `plain-gmail-`

### Test cases generation

- Create a directory for .eml files. Use the next structure as an example `__tests__/files/outlook-gmail/chrome-chrome/<test-case-dir>`.
- Put `sent.eml` and `received.eml` files into the `<test-case-dir>`
  - `sent.eml` contains MIME string logged by integrator application during the sending routine.
  - `received.eml` contains MIME string extracted manually from email client or logged by the integrator application.
    
    For outlook.com - extract MIME string by logging it in the application itself.
    _Do not do export MIME via the web client using `Show message source` action.
    It breaks spacing of the quoted-printable parts and parsing/validation of such MIME might fail._
    
    For mail.google.com - use `Show original` -> `Download original` actions in web client.
    _Don't copy contents provided by just `Show original` action because they miss base64 strings of the attachments_
- Create three test suites to cover html, plain and htmltext (pseudoplain) normalisation. E.g:
  - `html-outlook-gmail.test.ts`
  - `plain-outlook-gmail.test.ts`
  - `htmltext-outlook-gmail.test.ts`
- For every test suite, use built in functions to automate running of test cases.
  ```
  const testsPath = path.resolve(__dirname, `./files/outlook-gmail`);

  // in html-outlook-gmail.test.ts
  const runTests = createHtmlTestsRunner(testsPath, EMAIL_VENDORS.OUTLOOK);
  describe("Chrome-Chrome", runTests("chrome-chrome"));
  ...
  // in plain-outlook-gmail.test.ts
  const runTests = createPlainTestsRunner(testsPath);
  describe("Chrome-Chrome", runTests("chrome-chrome"));
  ...
  // in htmltext-outlook-gmail.test.ts
  const runTests = createPseudoplainTestsRunner(testsPath, EMAIL_VENDORS.OUTLOOK);
  describe("Chrome-Chrome", runTests("chrome-chrome"));
  ```

### Important clues

- MIME normaliser does not cover all provided test cases. Some of them pending to be fixed and ignored explicitly.  
  Example:

  ```javascript
  describe(
    "MacOS-MacOS",
    runTests("macos-macos", { ignore: ["03", "04", "05", "08", "09"] })
  );
  ```

  _Details about valid/failing cases presented in [coverage](#coverage) section._

- To debug specific test cases, please first isolate a desired test file using the regexp in watch mode
  and then use `checkExclusively` filter of the describe function.

  ```javascript
  describe(
    "MacOS-MacOS",
    runTests("macos-macos", { checkExclusively: ["03", "08", "09"] })
  );
  ```

- Tests are using both JSDOM and custom Vereign [DOM parser](https://code.vereign.com/code/js-toolbox/gsdom).
  Vereign DOM parser has been developed to support MIME normalisation in Google Add-on and is a must for testing. Whenever you develop a new normalisation logic, ensure that Vereign DOM parser support functions that you are applying.

### Coverage

- GMail-GMail
  - [Chrome-Chrome](__tests__/files/gmail-gmail/chrome-chrome/README.md)
- GMail-Outlook
  - [Chrome-Chrome](__tests__/files/gmail-outlook/chrome-chrome/README.md)
- Outlook-GMail
  - [Chrome-Chrome](__tests__/files/outlook-gmail/chrome-chrome/README.md)
- Outlook-Outlook
  - [Chrome-Chrome](__tests__/files/outlook-outlook/chrome-chrome/README.md)
  - [MacOS Desktop-MacOS Deskop](__tests__/files/outlook-outlook/macos-macos/README.md)
  - [Windows Desktop-Windows Deskop](__tests__/files/outlook-outlook/windows-windows/README.md)
