"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cloneAnchorFromPane = exports.pruneElement = exports.normalizeDocumentCommon = exports.ATTRIBUTES_TO_KEEP = exports.ELEMENT_TYPES_TO_REMOVE = void 0;
// this is a Node module. require is a must to work across different envs
const URL = require("url-parse");
exports.ELEMENT_TYPES_TO_REMOVE = {
    BR: true,
    HR: true,
    USE: true,
    SVG: true,
};
exports.ATTRIBUTES_TO_KEEP = {
    alt: true,
    src: true,
    cite: true,
    data: true,
    datetime: true,
    href: true,
    value: true,
};
const normalizeDocumentCommon = (body) => {
    /**
     * Unwind Outlook safelink wrappers
     */
    const anchors = body.getElementsByTagName("a");
    for (const anchor of anchors) {
        const url = new URL(anchor.getAttribute("href"), true);
        if (url.host.includes("safelinks.protection.outlook.com")) {
            anchor.setAttribute("href", url.query["url"]);
        }
    }
    /**
     * Unwind Gmail "googleusercontent" wrappers
     */
    const images = body.getElementsByTagName("img");
    for (const image of images) {
        let url;
        try {
            url = new URL(image.getAttribute("src"));
        }
        catch (e) {
            url = null;
        }
        if (url && url.host.includes("googleusercontent.com")) {
            const originalUrl = url.hash.slice(1, url.hash.length);
            if (originalUrl) {
                image.setAttribute("src", originalUrl);
            }
        }
    }
};
exports.normalizeDocumentCommon = normalizeDocumentCommon;
/**
 * Decides whether node should be removed
 * @param element
 */
const pruneElement = (element) => {
    return !!exports.ELEMENT_TYPES_TO_REMOVE[element.nodeName];
};
exports.pruneElement = pruneElement;
const cloneAnchorFromPane = (a, pane) => {
    try {
        const url = new URL(a.getAttribute("href"));
        // If this is external url
        if (url.host && url.protocol) {
            pane.parentNode.insertBefore(a.cloneNode(false), pane);
        }
    }
    catch (e) {
        return e;
    }
};
exports.cloneAnchorFromPane = cloneAnchorFromPane;
