"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanupHiddenCharacters = exports.normalizePlainPart = void 0;
// this is a Node module. require is a must to work across different envs
const URL = require("url-parse");
const utils_1 = require("../utils");
const normalizePlainPart = (text) => {
    text = exports.cleanupHiddenCharacters(text);
    text = removeQRCodes(text);
    text = utils_1.normalizeTextSpacings(text);
    text = patchOutlookSafelinksWrappers(text);
    return text.trim();
};
exports.normalizePlainPart = normalizePlainPart;
const patchOutlookSafelinksWrappers = (text) => {
    const links = text.match(/<https:.+?(safelinks\.protection\.outlook\.com).+?>/g);
    if (links) {
        links.forEach((link) => {
            const url = new URL(link.slice(1, link.length - 1), true);
            const originalUrl = url.query["url"];
            text = text.replace(link, `<${originalUrl}>`);
        });
    }
    return text;
};
/**
 * Function removes seal from the plain text.
 * Function has to support the next possible file names of the seal:
 * qrcode.png, qrcode-xxx.png, seal-xxx.png
 * @param s
 */
const removeQRCodes = (s) => {
    return s
        .replace(/\[(image:\s)*(qrcode|seal).*?.png]\s*<https:\/\/.+(vereign\.com|vrgnservices\.com).*?>/g, "")
        .replace(/<https:\/\/.+(vereign\.com|vrgnservices\.com).*?>\s*\[(image: )*(qrcode|seal).*?.png]/g, "");
};
const cleanupHiddenCharacters = (s) => {
    const removeSymbols = new RegExp(/[\u200B]+/g);
    return s.replace(removeSymbols, "");
};
exports.cleanupHiddenCharacters = cleanupHiddenCharacters;
